@@ -29,7 +29,7 @@
 #include <string.h>
 
 #include "gtkprogressbar.h"
-#include "gtkorientable.h"
+#include "gtkorientableprivate.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 
@@ -70,20 +70,17 @@
 
 struct _GtkProgressBarPrivate
 {
-  GtkOrientation orientation;
-
   gchar         *text;
 
   gdouble        fraction;
   gdouble        pulse_fraction;
 
-  guint          blocks;
-  gint           in_block;
-
   gint           activity_pos;
   guint          activity_blocks;
   guint          activity_step;
 
+  GtkOrientation orientation;
+
   guint          activity_dir  : 1;
   guint          activity_mode : 1;
   guint          ellipsize     : 3;
@@ -179,6 +176,16 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
                                                         NULL,
                                                         GTK_PARAM_READWRITE));
 
+  /**
+   * GtkProgressBar:show_text:
+   *
+   * Sets whether the progressbar will show text superimposed
+   * over the bar. The shown text is either the value of
+   * the #GtkProgressBar::text property or, if that is %NULL,
+   * the #GtkProgressBar::fraction value, as a percentage.
+   *
+   * Since: 3.0
+   */
   g_object_class_install_property (gobject_class,
                                    PROP_SHOW_TEXT,
                                    g_param_spec_boolean ("show-text",
@@ -289,8 +296,6 @@ gtk_progress_bar_init (GtkProgressBar *pbar)
                                             GtkProgressBarPrivate);
   priv = pbar->priv;
 
-  priv->blocks = 10;
-  priv->in_block = -1;
   priv->orientation = GTK_ORIENTATION_HORIZONTAL;
   priv->inverted = FALSE;
   priv->pulse_fraction = 0.1;
@@ -1035,8 +1040,7 @@ gtk_progress_bar_set_activity_mode (GtkProgressBar *pbar,
       if (priv->activity_mode)
         gtk_progress_bar_act_mode_enter (pbar);
 
-      if (gtk_widget_is_drawable (GTK_WIDGET (pbar)))
-        gtk_widget_queue_resize (GTK_WIDGET (pbar));
+      gtk_widget_queue_resize (GTK_WIDGET (pbar));
     }
 }
 
@@ -1106,8 +1110,7 @@ gtk_progress_bar_set_text (GtkProgressBar *pbar,
   g_free (priv->text);
   priv->text = text && *text ? g_strdup (text) : NULL;
 
-  if (gtk_widget_is_drawable (GTK_WIDGET (pbar)))
-    gtk_widget_queue_resize (GTK_WIDGET (pbar));
+  gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
   g_object_notify (G_OBJECT (pbar), "text");
 }
@@ -1140,8 +1143,7 @@ gtk_progress_bar_set_show_text (GtkProgressBar *pbar,
     {
       priv->show_text = show_text;
 
-      if (gtk_widget_is_drawable (GTK_WIDGET (pbar)))
-        gtk_widget_queue_resize (GTK_WIDGET (pbar));
+      gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
       g_object_notify (G_OBJECT (pbar), "show-text");
     }
@@ -1198,9 +1200,9 @@ gtk_progress_bar_set_orientation (GtkProgressBar *pbar,
   if (priv->orientation != orientation)
     {
       priv->orientation = orientation;
+      _gtk_orientable_set_style_classes (GTK_ORIENTABLE (pbar));
 
-      if (gtk_widget_is_drawable (GTK_WIDGET (pbar)))
-        gtk_widget_queue_resize (GTK_WIDGET (pbar));
+      gtk_widget_queue_resize (GTK_WIDGET (pbar));
     }
 }
 
@@ -1226,8 +1228,7 @@ gtk_progress_bar_set_inverted (GtkProgressBar *pbar,
     {
       priv->inverted = inverted;
 
-      if (gtk_widget_is_drawable (GTK_WIDGET (pbar)))
-        gtk_widget_queue_resize (GTK_WIDGET (pbar));
+      gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
       g_object_notify (G_OBJECT (pbar), "inverted");
     }
