@@ -22,9 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#undef GTK_DISABLE_DEPRECATED
-
-#include <gtk/gtk.h>
+#include <gtk/gtkx.h>
 
 #include "gailtoplevel.h"
 
@@ -96,8 +94,10 @@ gail_toplevel_init (GailToplevel *toplevel)
       if (!window || 
           !gtk_widget_get_visible (widget) ||
           is_attached_menu_window (widget) ||
-          GTK_WIDGET (window)->parent ||
-          GTK_IS_PLUG (window))
+#ifdef GDK_WINDOWING_X11
+          GTK_IS_PLUG (window) ||
+#endif
+          gtk_widget_get_parent (GTK_WIDGET (window)))
         {
           GList *temp_l  = l->next;
 
@@ -215,20 +215,16 @@ gail_toplevel_show_event_watcher (GSignalInvocationHint *ihint,
     return TRUE;
 
   widget = GTK_WIDGET (object);
-  if (widget->parent || 
+  if (gtk_widget_get_parent (widget) ||
       is_attached_menu_window (widget) ||
-      is_combo_window (widget) ||
-      GTK_IS_PLUG (widget))
+#ifdef GDK_WINDOWING_X11
+      GTK_IS_PLUG (widget) ||
+#endif
+      is_combo_window (widget))
     return TRUE;
 
   child = gtk_widget_get_accessible (widget);
-  if (!strcmp (atk_role_get_name (atk_object_get_role (child)), "redundant object"))
-    {
-      return TRUE;
-    }
-
-  child = gtk_widget_get_accessible (widget);
-  if (!strcmp (atk_role_get_name (atk_object_get_role (child)), "redundant object"))
+  if (atk_object_get_role (child) == ATK_ROLE_REDUNDANT_OBJECT)
     {
       return TRUE;
     }
@@ -237,21 +233,23 @@ gail_toplevel_show_event_watcher (GSignalInvocationHint *ihint,
    * Add the window to the list & emit the signal.
    * Don't do this for tooltips (Bug #150649).
    */
-  if (atk_object_get_role (child) != ATK_ROLE_TOOL_TIP)
-  {
-      toplevel->window_list = g_list_append (toplevel->window_list, widget);
+  if (atk_object_get_role (child) == ATK_ROLE_TOOL_TIP)
+    {
+      return TRUE;
+    }
 
-      n_children = g_list_length (toplevel->window_list);
+  toplevel->window_list = g_list_append (toplevel->window_list, widget);
 
-      /*
-       * Must subtract 1 from the n_children since the index is 0-based
-       * but g_list_length is 1-based.
-       */
-      atk_object_set_parent (child, atk_obj);
-      g_signal_emit_by_name (atk_obj, "children-changed::add",
-			     n_children - 1, 
-			     child, NULL);
-  }
+  n_children = g_list_length (toplevel->window_list);
+
+  /*
+   * Must subtract 1 from the n_children since the index is 0-based
+   * but g_list_length is 1-based.
+   */
+  atk_object_set_parent (child, atk_obj);
+  g_signal_emit_by_name (atk_obj, "children-changed::add",
+                         n_children - 1, 
+                         child, NULL);
 
   /* Connect destroy signal callback */
   g_signal_connect (G_OBJECT(object), 
@@ -325,9 +323,10 @@ _gail_toplevel_remove_child (GailToplevel *toplevel,
 static gboolean
 is_attached_menu_window (GtkWidget *widget)
 {
-  GtkWidget *child = GTK_BIN (widget)->child;
+  GtkWidget *child;
   gboolean ret = FALSE;
 
+  child = gtk_bin_get_child (GTK_BIN (widget));
   if (GTK_IS_MENU (child))
     {
       GtkWidget *attach;
@@ -344,26 +343,26 @@ is_attached_menu_window (GtkWidget *widget)
 static gboolean
 is_combo_window (GtkWidget *widget)
 {
-  GtkWidget *child = GTK_BIN (widget)->child;
+  GtkWidget *child;
   AtkObject *obj;
-  GtkAccessible *accessible;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
 
   if (!GTK_IS_EVENT_BOX (child))
     return FALSE;
 
-  child = GTK_BIN (child)->child;
+  child = gtk_bin_get_child (GTK_BIN (child));
 
   if (!GTK_IS_FRAME (child))
     return FALSE;
 
-  child = GTK_BIN (child)->child;
+  child = gtk_bin_get_child (GTK_BIN (child));
 
   if (!GTK_IS_SCROLLED_WINDOW (child))
     return FALSE;
 
   obj = gtk_widget_get_accessible (child);
   obj = atk_object_get_parent (obj);
-  accessible = GTK_ACCESSIBLE (obj);
 
   return  FALSE;
 }
